{****************************************************************************
** Background timer routines                                               **
**  by Steven H Don                                                        **
*****************************************************************************
** This shows you how to speed up the system timer to specific frequencies **
** and use it do do some useful stuff.                                     **
** NOTE: Windows doesn't like timers at speeds of more than 1000Hz         **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************}
program TimerDemo;

uses DOS, Crt;

var
  {This stores a pointer to the original timer handler}
  BIOSTimerHandler : procedure;
  {This is used for counting the clock ticks}
  ClockTicks, TickCounter : LongInt;

{This procedure changes the speed of the timer}
procedure ChangeSpeed (Divisor : LongInt);
begin
  {Set the appropriate values for the timer interrupt}
  TickCounter := $1234DD div Divisor;
  Port[$43] := $34;
  Port[$40] := lo (TickCounter);
  Port[$40] := hi (TickCounter);
end;

{This is the new timer interrupt handler}
{$F+}
procedure TimerHandler; interrupt;
begin
  {Do whatever you want here, I'm just displaying a character}
  Write ('!');
  {Check whether we need to call the original timer handler}
  ClockTicks := ClockTicks + TickCounter;
  {Do so if required}
  if ClockTicks > 65535 then begin
    dec (ClockTicks, 65536);
    asm pushf end;
    BIOSTimerHandler;
  end else
    Port [$20] := $20;
end;
{$F-}

{Installs the timer handler}
procedure InstallTimer;
begin
  TickCounter := 65536;
  {Install new timer handler}
  SetIntVec(8, Addr(TimerHandler));
end;

{Restores the BIOS timer handler}
procedure RestoreTimer;
begin
  {Return to 18.2 times a second}
  Port[$43] := $34;
  Port[$40] := 0;
  Port[$40] := 0;
  {Install old timer handler}
  SetIntVec(8, @BIOSTimerHandler);
end;

begin
  {Save old timer handler}
  GetIntVec(8, @BIOSTimerHandler);
  {Do the demo}
  ClrScr;
  Writeln ('Press any key to install new timer handler.');
  ReadKey;
  InstallTimer;
  Writeln ('Timer installed.');
  ReadKey;
  Writeln;
  Writeln ('Increasing speed to 100Hz');
  ChangeSpeed (100);
  ReadKey;
  RestoreTimer;
  Writeln;
  Writeln ('Timer restored to BIOS routine.');
  ReadKey;
end.